th {
    color: #6D929B;
}
td {
    white-space:nowrap;
}
td.address {
    color:gray;
    font-family: monospace;
}
span.memory {
    color:blue;
    white-space: nowrap;
    cursor:default;
}

table {
    border-width:10px;
}

span.perms {
    color:gray;
    font-family: monospace;
}
td.perms {
    font-family: monospace;
}
td.memory, th.memory {
    text-align: right;
    white-space: nowrap;
}
td.comment {
    color:gray;
}
div.innertoc {
    float:right;
    background-color: #eeeeee;
    padding:5px;
}
div.summaryTable {
    float:left;
    padding:0px;
    min-width:48%;
}
thead th {
    border-color: gray;
    border-style: solid;
    border-width: 0px 0px 1px 0px;
}
table {
    margin-left: auto;
    margin-right: auto;
}
table.sortable th {
    cursor:default;
}
tbody.hidden {
    display:none;
}
div.fullDetails {
    clear:both;
    float:left;
}
table.fullDetails {
    display:none;
    width: 100%;
}
table.librarySummary {
    width: 48%;
}

a.page {
    display:block;
    text-align: right;
}
td {
    padding: 0em 0.3em;
}
tfoot.showFullLibrarySummaryLink {
    font-size: small;
    text-align: right;
}

span.definedWord {
    border-bottom: dashed 1px blue;
    cursor:help;
}
th.definedWord {
    cursor:help;
}

span.errorMessage {
    color: red;
}
